#include <iostream>
#include <vector>
#include <map>
#include <utility>
#include <numeric>
#include <limits>
#include <cmath>
#include <TSystem.h>

#include "filval/filval.hpp"
#include "filval/root/filval.hpp"

#include <boost/range/combine.hpp>
#include <boost/format.hpp>

#include "analysis/TrackingNtuple.h"
#include "analysis/TrackingNtupleObjs.hpp"
#include "analysis/common.hpp"

using namespace std;
using namespace fv;
using namespace fv::root;

typedef std::pair<std::vector<float>,std::vector<float>> pair_vec;

#define HIT_TYPE_PIXEL 0
#define HIT_TYPE_GLUED 1
#define HIT_TYPE_STRIP 2

enum PixSubDet{
    Void = 0,
    Barrel = 1,
    EndCap = 2
};

struct PixDetElem{
    PixSubDet subdet;
    unsigned char layer;

    PixDetElem():subdet(PixSubDet::Void), layer(0) {}

    PixDetElem(const PixSubDet& subdet, const unsigned char& layer){
        this->subdet = subdet;
        this->layer = layer;
    }

    template<typename T>
    bool contains(const T& obj) const{
        return obj.subdet() == subdet && obj.layer() == layer;
    }

    bool contains(const PixSubDet& subdet, const unsigned char& layer) const{
        return subdet == this->subdet && layer == this->layer;
    }
};

std::map<PixSubDet,string> subdet_names = {{PixSubDet::Barrel, "BPIX"}, {PixSubDet::EndCap, "FPIX"}};


typedef std::tuple<PixRecHit, SimHit> HitPair;

vector<string> seedTypes =
    {"initialStepSeeds",
     "highPtTripletStepSeeds",
     "mixedTripletStepSeeds",
     "pixelLessStepSeeds",
     "tripletElectronSeeds",
     "pixelPairElectronSeeds",
     "stripPairElectronSeeds"};

SeedCollection seeds;
SimTrackCollection sim_tracks;
SimVertexCollection sim_vertices;

void register_objects(TrackingDataSet& tds){
    seeds.init(tds);
    sim_tracks.init(tds);
    sim_vertices.init(tds);
}

float rad(const float& x, const float& y) {
    return sqrt(x*x + y*y);
}

void run_analysis(const vector<fv::util::DataFileDescriptor>& dfds, const string output_filename, bool silent){
    TrackingDataSet tds(output_filename, dfds, "trackingNtuple/tree");
    register_objects(tds);

    // Indices of prompt gen electrons
    auto prompt_electrons = func_value<vector<size_t>>("prompt_electrons",
        FUNC(([](){
            vector<size_t> idxs;
            for(const auto& sim_track : sim_tracks) {
                if (abs(sim_track.pdgId()) != 11) continue;
                const SimVertex& sim_vertex = sim_vertices[sim_track.parentVtxIdx()];
                float rho = rad(sim_vertex.x(), sim_vertex.y());
                float z = abs(sim_vertex.z());
                if (rho > 1 or z > 10) continue;
                idxs.push_back(sim_track.idx);
            }
            return idxs;
        })));

    // Indices of prompt gen electrons that have a matched seed
    function<bool(size_t)> with_seed = [](const size_t& e_idx) {
        if (seeds.size() == 0 or
            sim_tracks[e_idx].seedIdx().size() == 0) return false;
        else return true;
    };

    // Indices of prompt gen electrons that have a matched gsftrack
    function<bool(size_t)> with_track = [](const size_t& e_idx) {
        return sim_tracks[e_idx].trkIdx().size() > 0;
    };

    function<float(size_t)> e_pt = [](const size_t& e_idx) {
        return sim_tracks[e_idx].pt();
    };

    function<float(size_t)> e_eta = [](const size_t& e_idx) {
        return sim_tracks[e_idx].eta();
    };

    function<float(size_t)> e_phi = [](const size_t& e_idx) {
        return sim_tracks[e_idx].phi();
    };

    tds.register_container<EfficiencyContainer<size_t>>("seed_eff_v_pt",
        prompt_electrons, TH1Params::lookup("eff_v_pt"), nullptr, &with_seed, &e_pt);

    tds.register_container<EfficiencyContainer<size_t>>("seed_eff_v_eta",
        prompt_electrons, TH1Params::lookup("eff_v_eta"), nullptr, &with_seed, &e_eta);

    tds.register_container<EfficiencyContainer<size_t>>("seed_eff_v_phi",
        prompt_electrons, TH1Params::lookup("eff_v_phi"), nullptr, &with_seed, &e_phi);

    tds.register_container<EfficiencyContainer<size_t>>("track_eff_v_pt",
        prompt_electrons, TH1Params::lookup("eff_v_pt"), nullptr, &with_track, &e_pt);

    tds.register_container<EfficiencyContainer<size_t>>("track_eff_v_eta",
        prompt_electrons, TH1Params::lookup("eff_v_eta"), nullptr, &with_track, &e_eta);

    tds.register_container<EfficiencyContainer<size_t>>("track_eff_v_phi",
        prompt_electrons, TH1Params::lookup("eff_v_phi"), nullptr, &with_track, &e_phi);

    // Indices of ecal-driven seeds
    auto ecal_seeds = func_value<vector<size_t>>("ecal_seeds",
        FUNC(([](){
            vector<size_t> idxs;
            for(const auto& seed : seeds) {
                if (seed.ecalDriven())
                    idxs.push_back(seed.idx);
            }
            return idxs;
        })));

    function<bool(size_t)> with_sim_track = [](const size_t& seed_idx) {
        for (const int& simTrkIdx : seeds[seed_idx].simTrkIdx()) {
            const SimTrack& sim_track = sim_tracks[simTrkIdx];
            if (abs(sim_track.pdgId()) != 11) continue;
            const SimVertex& sim_vertex = sim_vertices[sim_track.parentVtxIdx()];
            float rho = rad(sim_vertex.x(), sim_vertex.y());
            float z = abs(sim_vertex.z());
            if (rho > 1 or z > 10) continue;
            return true;
        }
        return false;
    };

    function<float(size_t)> seed_pt = [](const size_t& seed_idx) {
        return seeds[seed_idx].pt();
    };

    function<float(size_t)> seed_eta = [](const size_t& seed_idx) {
        return seeds[seed_idx].eta();
    };

    function<float(size_t)> seed_phi = [](const size_t& seed_idx) {
        return seeds[seed_idx].phi();
    };

    tds.register_container<EfficiencyContainer<size_t>>("seed_pur_v_pt",
        ecal_seeds, TH1Params::lookup("pur_v_pt"), nullptr, &with_sim_track, &seed_pt);

    tds.register_container<EfficiencyContainer<size_t>>("seed_pur_v_eta",
        ecal_seeds, TH1Params::lookup("pur_v_eta"), nullptr, &with_sim_track, &seed_eta);

    tds.register_container<EfficiencyContainer<size_t>>("seed_pur_v_phi",
        ecal_seeds, TH1Params::lookup("pur_v_phi"), nullptr, &with_sim_track, &seed_phi);

    tds.process(silent);
    tds.save_all();
}

int main(int argc, char * argv[]){
    fv::util::ArgParser args(argc, argv);
    bool silent = args.cmdOptionExists("-s");
    if(args.cmdOptionExists("-c")) {
        fv::util::init_config(args.getCmdOption("-c"));
        auto output_filename = fv::util::the_config->get_output_filename();
        if (output_filename == "") return -1;

        fv::util::init_log(fv::util::LogPriority::kLogInfo);
        gSystem->Load("libfilval.so");

        auto file_list = fv::util::the_config->get_source_files();
        run_analysis(file_list, output_filename, silent);
    } else {
        cout << "Usage: ./tracking_eff (-s) -c config_file.yaml" << endl;
    }
    return 0;
}
